from bclearer_core.infrastructure.nf.objects.registries.nf_verse_registries import (
    NfVerseRegistries,
)


class NfUniverseRegistries(
    NfVerseRegistries
):
    """Universe-level registries for verses."""
