# TODO Circular reference to be fixed - super class to be added
# TODO: fix setup for configurations with single called imports - cannot change after setup
class BieIdConfigurations:
    EXPORT_INT_VALUE_COLUMNS = False
