from enum import auto

from bclearer_orchestration_services.identification_services.b_identity_ecosystem.common_knowledge.bie_enums import (
    BieEnums,
)


class AppRunSessionObjectColumnNames(
    BieEnums
):
    NOT_SET = auto()

    START_DATE_TIMES = auto()

    OWNING_OS_USER_PROFILE_SESSION_BIE_IDS = (
        auto()
    )
