from bclearer_orchestration_services.identification_services.b_identity_ecosystem.common_knowledge.bie_infrastructure_types import (
    BieInfrastructureTypes,
)


class BieAppRunSessionTypes(
    BieInfrastructureTypes
):
    pass
