from bclearer_core.configurations.b_clearer_configurations.b_clearer_configurations import (
    BClearerConfigurations,
)


class BClearerDomainConfigurations(
    BClearerConfigurations
):
    pass
