from bclearer_core.infrastructure.nf.objects.registers.nf_universe_registers import (
    NfUniverseRegisters,
)


class NfDomainUniverseRegisters(
    NfUniverseRegisters
):
    def __init__(self) -> None:
        super().__init__()
