from bclearer_core.infrastructure.nf.objects.registries.nf_universe_registries import (
    NfUniverseRegistries,
)


class NfDomainUniverseRegistries(
    NfUniverseRegistries
):
    def __init__(self) -> None:
        super().__init__()
