from bclearer_core.infrastructure.nf.objects.registers.nf_verse_registers import (
    NfVerseRegisters,
)


class NfUniverseRegisters(
    NfVerseRegisters
):
    """Universe-level verse registers."""
