import os

WRITE_ACRONYM = "w"

UTF_8_ENCODING_NAME = "utf-8"

DEFAULT_NULL_VALUE = "DefaultNullValue"

FOLDER_DELIMITER = os.sep

CLONED_DATA_FOLDER_NAME = "cloned_data"

LOCAL_GIT_REPOSITORY_FOLDER_NAME = (
    "local_git_repositories"
)

LOCAL_CONFIGURATION_FOLDER = (
    "local_configuration"
)

LOCAL_CONFIGURATION_FILE = (
    "local_configuration.json_service"
)

ORIGINAL_DATA_CONFIGURATION_FILE = "original_data_configuration.json_service"

CODE_FOLDER_NAME = "code"

SANDPIT_FOLDER_NAME = "sandpit"

EXECUTABLES_FOLDER_NAME = "executables"

PYTHON_COMMIT_MESSAGE = (
    "Python Commit - Code and CD"
)

B_UNIT_CONFIGURATION_FILE = (
    "b_unit_configuration.json_service"
)

PROJECT_SECTION_B_UNITS = "b_units"

B_UNIT_ATTRIBUTE_B_UNIT_NAME = "name"

B_UNIT_SECTION_INPUTS = "inputs"

INPUT_ATTRIBUTE_GIT_REPOSITORY = (
    "git_repository"
)

INPUT_ATTRIBUTE_GIT_RELEASE_GUID = (
    "release_guid"
)

INPUT_ATTRIBUTE_FILE_NAME = "file_name"

CHECK_RESULTS_DATAFRAME_NAME_SUCCESS_SUFFIX = (
    "success"
)

CHECK_RESULTS_DATAFRAME_NAME_FAILURE_SUFFIX = (
    "failure"
)

NAME_COMPONENT_DIVIDER = "_"

HR_NAME_COMPONENT_DIVIDER = "_"

HR_DATE_TIME_FORMAT = "%Y%m%d_%H%M"

DEFAULT_MASTER_TABLE_SUFFIX = "_1"

DEFAULT_FOREIGN_TABLE_SUFFIX = "_2"
