from bclearer_core.configurations.b_clearer_configurations.b_clearer_domain_configurations import (
    BClearerDomainConfigurations,
)


class BClearerDomainGraphConfigurations(
    BClearerDomainConfigurations
):
    SOURCE_GRAPH_FILE_FORMAT = str()
